SMUS_CLIENT_INFO = "AmazonQ-For-SMUS-IDE"

Q_UNSUBSCRIBED_MESSAGE = "You are not subscribed to Amazon Q Developer. Please contact your domain admin to activate your subscription."
INTERAL_SERVER_ERROR_MESSAGE = "Something went wrong. Please try again later."
LOG_FILE_NAME = "sagemaker_genai_jl_extension.log"
LOGGER_NAME = "sagemaker-genai-jupyterlab-ext-operations"
LOG_FILE_PATH = "/var/log/studio/sagemaker_genai_jl_ext"
METRICS_NAMESPACE = "SagemakerGenAIJupyterLab"