export const FLARE_IFRAME_ID = 'flare-iframe';
export const TOGGLE_AI_CHAT_MESSAGE = 'MD_TOGGLE_AI_CHAT';
export const WORKSPACE_ROOT_PATH = '/home/sagemaker-user';
export const DEFAULT_CURSOR_STATE = { position: { line: 0, character: 0 } };

export const QUICK_ACTIONS = {
  HELP: '/help',
  FIX: '/fix',
  EXPLAIN: '/explain',
  OPTIMIZE: '/optimize',
  REFACTOR: '/refactor'
} as const;
